/*
  This sketch aims to set up a lookup table or some translation table or whatever its called I can use for my display.
  The display is mostly 19 columns of 7 LED's. At start there is 1 column of 1 LED and 1 of 5 LEDs which is mirrored at the end. These are all in one LED chain, arranged antiparallel. This is a real bitch to program for if you want to animate things, move letters (sprites) around, etc as it is a weird coordinate system.
  The idea is that I will have an easier coordinate system 'virtual grid' of LEDs which are arranged in 23 columns of 9 LEDs (so there is one row of LEDs below and above 'overscan') which I will use to position a LED (or LED 'sprite') on.
  The 'virtual grid' has the origin x,y 0,0 at topleft; this is virtLED 0. The virtLED number increments first horizontally to virtLED 22 (coordinate 22,0; virtLED 24 is the start of row 2 (coordinate 0,1), this continues in TV-like scanlines all the way to coordinate (23,9) corresponds to virtLED number 207

  The virtLED positions then need to be converted to the actual LED strip positions to light up. Could be done using dome finicky formula plus some exceptions, or just a lookup table with 2 columns, one the virtLED input and two the realLED output. The realLED values are then sent to the WS2812 strip.

  So the virtLED grid then needs to be translated to the strip as below. Aanything in the overscan area should be ignored or translate to a value of NUM_LEDS+1 so it doesnt actually display.

  virtLED                             realLED
  0   1   2   3   4   5   6  etc..          x  x  x  x  x  x  x  x (overscan line)
  25  26  27  28  29  30  31                x  x  12 13 26 27 40 41
  50  51  52  53  54  55  56      ==>       x  1  11 14 25 28 39 42
  75  76  77  78  79  80  81                x  2  10 15 24 29 38 43
  100 101 102 103 104 105 106               0  3  9  16 23 30 37 44
  etc..

  So I can move a sprite in any direction in this way, depending on the direction I add or subtract an integer:

  -24      -23      -22
          up
  -1  left  0  right +1
        down
  +22      +23      +24
*/

#include <FastLED.h>
#define NUM_LEDS 145
#define DATA_PIN 6
#define BRIGHTNESS  100
CRGB leds[NUM_LEDS];

int LEDpos = 0;
byte realLED[] = {145, 145, 145, 145, 145, 145, 145, 145, 145, 145, 145, 145, 145, 145, 145, 145, 145, 145, 145, 145, 145, 145, 145, 145, 145, 12, 13, 26, 27, 40, 41, 54, 55, 68, 69, 82, 83, 96, 97, 110, 111, 124, 125, 138, 145, 145, 145, 1, 11, 14, 25, 28, 39, 42, 53, 56, 67, 70, 81, 84, 95, 98, 109, 112, 123, 126, 137, 139, 145, 145, 2, 10, 15, 24, 29, 38, 43, 52, 57, 66, 71, 80, 85, 94, 99, 108, 113, 122, 127, 136, 140, 145, 0, 3, 9, 16, 23, 30, 37, 44, 51, 58, 65, 72, 79, 86, 93, 100, 107, 114, 121, 128, 135, 141, 144, 145, 4, 8, 17, 22, 31, 36, 45, 50, 59, 64, 73, 78, 87, 92, 101, 106, 115, 120, 129, 134, 142, 145, 145, 5, 7, 18, 21, 32, 35, 46, 49, 60, 63, 74, 77, 88, 91, 102, 105, 116, 119, 130, 133, 143, 145, 145, 145, 6, 19, 20, 33, 34, 47, 48, 61, 62, 75, 76, 89, 90, 103, 104, 117, 118, 131, 132, 145, 145, 145, 145, 145, 145, 145, 145, 145, 145, 145, 145, 145, 145, 145, 145, 145, 145, 145, 145, 145, 145, 145, 145, 145
                 };


void setup() {
  Serial.begin(9600);
  FastLED.addLeds<WS2812B, DATA_PIN, GRB>(leds, NUM_LEDS); //This tells the library that there is a strand of WS2812's on DATA_PIN
  FastLED.setBrightness(BRIGHTNESS);
  FastLED.clear();
}

void loop() {
  delay(300);
  striporder();
  FastLED.clear();
  gridorder();
  FastLED.clear();
  right_down();
  left_down();
  left_up();
  right_up();
}



void striporder() { //This illuminates the leds in the order they are on the strip, so in order of realLED position
  for (byte i = 0; i <= NUM_LEDS; i++) {
    leds[i] = CHSV (2 * i, 255, 255);
    FastLED.show();
    //    delay (10);
  }
}

void gridorder() {
  for (byte i = 0; i <= 206; i++) { //Use the realLED array to go through all 207 virtual LEDs bij scanline, converting them to the realLED position
    byte r = realLED[i]; //Grab the realLED value at index i and set r to this value
    leds[r] = CHSV (r, 255, 255);
    FastLED.show();
    //    delay (10);
  }
}

void right_down() { //lights up diagonals to the right and down
  for (byte i = 0; i < 206; i++) {
    LEDpos = (i * 24) % 206; //increments of 24 will move the position down and right. Take the modulo after incrementing to make sure the code loops if i*24 gets over 206
    byte r = realLED[LEDpos]; //Grab the realLED value at index p and set r to this value
    leds[r] = CHSV (r * 2, 255, 255);
    FastLED.show();
    delay(100);
    leds[r] = CHSV (0, 0, 0);
  }
}

void left_down() { //lights up diagonals to the left and down
  for (byte i = 0; i < 206; i++) {
    LEDpos = (i * 22) % 206; //increments of 22 will move the position down and left. Take the modulo after incrementing to make sure the code loops if i*22 gets over 206
    byte r = realLED[LEDpos]; //Grab the realLED value at index p and set r to this value
    leds[r] = CHSV (r * 3, 255, 255);
    FastLED.show();
    delay (100);
    leds[r] = CHSV (0, 0, 0);
  }
}

void left_up() { //lights up diagonals to the left and down
  for (byte i = 206; i > 0; i--) {
    LEDpos = (i * 24) % 206;
    byte r = realLED[LEDpos];
    leds[r] = CHSV (r * 4, 255, 255);
    FastLED.show();
    delay (100);
    leds[r] = CHSV (0, 0, 0);
  }
}

void right_up() { //lights up diagonals to the left and down
  for (byte i = 206; i > 0; i--) {
    LEDpos = (i * 22) % 206;
    byte r = realLED[LEDpos];
    leds[r] = CHSV (r * 2, 255, 255);
    FastLED.show();
    delay (100);
    leds[r] = CHSV (0, 0, 0);
  }
}
